MISSING_SETUP = 'Missing implementation of "setup" in PactHelper'
MISSING_TEAR_DOWN = 'Missing implementation of "tear_down" in PactHelper'


class PactHelper(object):
    test_url = 'localhost'
    test_port = 9999
